import 'dart:typed_data';
import 'dart:convert';
import 'dart:math';

import '../lib/dartgl.dart';
import '../lib/dartgl_sdl.dart';

var arrays;
var textures;
var program;
var location;
const int COUNT = 10; 

void main() {
  print("SDL2 ${SDL_Init()}");  
  Pointer w = SDL_CreateWindow("OpenGL", 100, 100, 600, 400, SDL_WINDOW_OPENGL|SDL_WINDOW_RESIZABLE);
  Pointer c = SDL_GL_CreateContext(w);
  
  SDL_GL_MakeCurrent(w, c);
  SDL_GL_SetSwapInterval(0);
  
  prepare();
    
  double time = 0.0;
  int count = 0;
  var exit = false;

  var startTime = new DateTime.now();
  var event = new Event();
  while (!exit){
    
    while (SDL_PollEvent(event) != 0){
      if (event.type == 0x100) {
        print(event.type);
        exit = true;
      }
    }
    
    time += PI/64;
    count++;
    draw(time);
        
    SDL_GL_SwapWindow(w);
  }
  print(count/new DateTime.now().difference(startTime).inSeconds);
}


void draw(double time){
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  
    
  glUseProgram(program);
  glUniform1f(location, sin(time)*0.2 + 0.8);
  
  glActiveTexture(GL_TEXTURE0);
  glBindTexture(GL_TEXTURE_2D, textures[0]);
  glUniform1i(glGetUniformLocation(program, "texture1"), 0);
    
  glBindVertexArray(arrays[0]);          // select first VAO
  for (int i = 0; i < 100; i++){
    glDrawArrays(GL_TRIANGLES, 0, 6);      // draw first object  
  }
  
  glBindVertexArray(0);
}

void prepare(){

  glClearColor(0.0, 0.7, 0.99, 0.0);
  
  var vertexShader = glCreateShader(GL_VERTEX_SHADER);
  shader(vertexShader, v);    
  
  var fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
  shader(fragmentShader, f);   
  
  program = glCreateProgram();
  glAttachShader(program, vertexShader);
  glAttachShader(program, fragmentShader);
  
  glBindAttribLocation(program, 0, "in_Position");
  glBindAttribLocation(program, 1, "in_Color");
  glBindAttribLocation(program, 2, "in_Texture");
  
  glLinkProgram(program);
  location = glGetUniformLocation(program, "in_Time");
    
  arrays = new Uint32List(1);
  glGenVertexArrays(1, arrays);
    
  
  
  var data = new Float32List.fromList([1.0, 1.0, -1.0, -1.0, -1.0,-1.0, 1.0, -1.0, -1.0,
                                      1.0, 1.0, -1.0, -1.0, -1.0,-1.0, -1.0, 1.0, -1.0]); 
  var data2 = new Float32List.fromList([1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0,
                                        1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0]);
  var data3 = new Float32List.fromList([0.0, 0.0, 1.0, 1.0, 0.0, 1.0,
                                        0.0, 0.0, 1.0, 1.0, 1.0, 0.0]);
  
  var buffers = new Uint32List(3);
  glGenBuffers(3, buffers);
    
  glBindVertexArray(arrays[0]);
  
  glBindBuffer(GL_ARRAY_BUFFER, buffers[0]);
  glBufferData(GL_ARRAY_BUFFER, data.lengthInBytes, data, GL_STATIC_DRAW);
  
  glVertexAttribPointer(0, 3, GL_FLOAT, false, 0, null); 
  glEnableVertexAttribArray(0);
  
  glBindBuffer(GL_ARRAY_BUFFER, buffers[1]);
  glBufferData(GL_ARRAY_BUFFER, data2.lengthInBytes, data2, GL_STATIC_DRAW);
  glVertexAttribPointer(1, 3, GL_FLOAT, false, 0, null); 
  glEnableVertexAttribArray(1);
  
  glBindBuffer(GL_ARRAY_BUFFER, buffers[2]);
  glBufferData(GL_ARRAY_BUFFER, data3.lengthInBytes, data3, GL_STATIC_DRAW);
  glVertexAttribPointer(2, 2, GL_FLOAT, false, 0, null); 
  glEnableVertexAttribArray(2);
  
  textures = new Uint32List(1);
  glGenTextures(1, textures);
  
  glBindTexture(GL_TEXTURE_2D, textures[0]);
      
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 2, 0, GL_RGBA, GL_UNSIGNED_BYTE, new Uint32List.fromList([0,1000000,0,1000000]));
  
}

void shader(int shader, String text) {
  
  var lines = text.split("\n").map((l)=>l+"\n").toList();
  var lengths =  lines.map((l)=>l.length).toList();
  
  glShaderSource(shader, lines.length, lines, null);
  
  glCompileShader(shader);  
  
  var status = new Int32List(1);
  glGetShaderiv(shader, GL_COMPILE_STATUS, status);
  if (status[0] != GL_TRUE){
    print("shader status ${status[0]}");
    
    Int64List length = new Int64List(1);
    TypedData infoLog = new ByteData(256+1);
    glGetShaderInfoLog(shader, 256, length, infoLog);
    
    if (length[0] > 0){
      print(UTF8.decode(new Uint8List.view(infoLog.buffer)..toList())); 
    }     
  }    
}

void glInfo() {
  print("GL_VENDOR: ${glGetString(GL_VENDOR)}");
  print("GL_RENDERER: ${glGetString(GL_RENDERER)}");
  print("GL_VERSION: ${glGetString(GL_VERSION)}");
}

var v = """
#version 130

uniform float in_Time;
in vec3 in_Position;
in vec3 in_Color;
in vec2 in_Texture;
out vec3 ex_Color;
out vec2 ex_Texture;

void main(void)
{
  gl_Position = vec4(in_Position.x*in_Time, in_Position.y*in_Time, in_Position.z*in_Time, 1.0);
  ex_Color = vec3(in_Color.r, in_Color.g, in_Color.b);
  ex_Texture = in_Texture;
}
""";

var f = """
#version 130

precision highp float; // needed only for version 1.30

uniform sampler2D texture1;
in vec3 ex_Color;
in vec2 ex_Texture;
out vec4 out_Color;

void main(void)
{
  //out_Color = texture(texture1, ex_Texture).xyzw;
  out_Color = vec4(ex_Color,1.0);
}
""";